@@ -25,7 +25,7 @@ var NAVTREE =
         [ "XMLDIFF_OP op", "d9/d25/transapi.html#op", [
           [ "Valid combinations of events", "d9/d25/transapi.html#combinations", null ]
         ] ],
-        [ "xmlNodePtr node", "d9/d25/transapi.html#node", null ],
+        [ "xmlNodePtr old_node & new_node", "d9/d25/transapi.html#node", null ],
         [ "strict nc_err **error", "d9/d25/transapi.html#error", null ]
       ] ],
       [ "History of the transAPI versions", "d9/d25/transapi.html#transAPI-history", null ],
@@ -39,7 +39,6 @@ var NAVTREE =
       ] ]
     ] ],
     [ "Todo List", "dd/da0/todo.html", null ],
-    [ "Deprecated List", "da/d58/deprecated.html", null ],
     [ "Modules", "modules.html", "modules" ],
     [ "Data Structures", null, [
       [ "Data Structures", "annotated.html", "annotated" ],
@@ -67,9 +66,9 @@ var NAVTREE =
 var NAVTREEINDEX =
 [
 "annotated.html",
-"d5/d7d/group__url.html#ggaeecc86d0808c9098b6a8694e88925ccda68d344995eae71ef85e07bb4a97c13e1",
-"db/d52/group__session.html#ga6465c58b3ca057b46655d5ffa90caff3",
-"db/de9/group__rpc.html#ga7977ca8b562643607b3c846ed36bbb23"
+"d5/d7d/group__url.html#ggaeecc86d0808c9098b6a8694e88925ccda5977de49ac0680ca80381aeb77aa40e6",
+"db/d52/group__session.html#ga3868d5331db61a5d3a4df3ff5fa33851",
+"db/de9/group__rpc.html#ga599c715b0aee0f1f2ed231ce1d78e5bd"
 ];
 
 var SYNCONMSG = 'click to disable panel synchronisation';
