@@ -2,8 +2,8 @@ var searchData=
 [
   ['nc_5fcallback_5ferror_5freply',['nc_callback_error_reply',['../d0/de2/group__reply.html#ga771a110143440bcd3b7d4c0e98388e80',1,'callbacks.h']]],
   ['nc_5fcallback_5fprint',['nc_callback_print',['../d3/d35/group__gen_a_p_i.html#ga806dfa9c27d2b8076bae21bcd549cce7',1,'callbacks.h']]],
-  ['nc_5fcallback_5fssh_5fhost_5fauthenticity_5fcheck',['nc_callback_ssh_host_authenticity_check',['../db/d52/group__session.html#gaacafd4dbd19385fc395f965deaff0c86',1,'callbacks_ssh.h']]],
-  ['nc_5fcallback_5fsshauth_5finteractive',['nc_callback_sshauth_interactive',['../db/d52/group__session.html#gacb6042fe2f3cfe5d79e01de599c546eb',1,'callbacks_ssh.h']]],
+  ['nc_5fcallback_5fssh_5fhost_5fauthenticity_5fcheck',['nc_callback_ssh_host_authenticity_check',['../db/d52/group__session.html#ga78a69a69dea0369455218c5aeedce9f8',1,'callbacks_ssh.h']]],
+  ['nc_5fcallback_5fsshauth_5finteractive',['nc_callback_sshauth_interactive',['../db/d52/group__session.html#ga41c10d3af57cccebc7c67b11e4024c21',1,'callbacks_ssh.h']]],
   ['nc_5fcallback_5fsshauth_5fpassphrase',['nc_callback_sshauth_passphrase',['../db/d52/group__session.html#ga81ef9b1c1949bbffd86cbfd9aa159726',1,'callbacks_ssh.h']]],
   ['nc_5fcallback_5fsshauth_5fpassword',['nc_callback_sshauth_password',['../db/d52/group__session.html#ga28299d575ef4e234ed96bad5c4f086fb',1,'callbacks_ssh.h']]],
   ['nc_5fcallhome_5faccept',['nc_callhome_accept',['../db/de7/group__callhome.html#ga556966a9c54e814e02b4afadf128b9b7',1,'callhome.h']]],
@@ -25,6 +25,7 @@ var searchData=
   ['nc_5fcpblts_5fnew',['nc_cpblts_new',['../db/d52/group__session.html#gabdd04a64fd465ebbf910c6b08393ac43',1,'session.h']]],
   ['nc_5fcpblts_5fremove',['nc_cpblts_remove',['../db/d52/group__session.html#gacb59cf4fa78bf02c22afac395acf0693',1,'session.h']]],
   ['nc_5fdatetime2time',['nc_datetime2time',['../d3/d35/group__gen_a_p_i.html#ga4dfd796fda30d3ad8588b4c30c4c1583',1,'netconf.h']]],
+  ['nc_5fdel_5fkeypair_5fpath',['nc_del_keypair_path',['../db/d52/group__session.html#ga93972e2d0602d0f3fd2728af279e61c1',1,'callbacks_ssh.h']]],
   ['nc_5ferr_5fdup',['nc_err_dup',['../d3/d35/group__gen_a_p_i.html#ga316faef1140f1b48a2ed1de6d84d2bef',1,'error.h']]],
   ['nc_5ferr_5ffree',['nc_err_free',['../d3/d35/group__gen_a_p_i.html#gac4bd51febe24b517d1952d1c1d8f6b07',1,'error.h']]],
   ['nc_5ferr_5fget',['nc_err_get',['../d3/d35/group__gen_a_p_i.html#ga3d7714f1f56ac7203a80c61aea4344f8',1,'error.h']]],
@@ -84,6 +85,8 @@ var searchData=
   ['nc_5frpc_5fvalidate',['nc_rpc_validate',['../db/de9/group__rpc.html#ga14fa1c78362457a646d5d820206d5158',1,'messages.h']]],
   ['nc_5fsession_5faccept',['nc_session_accept',['../db/d52/group__session.html#gad0e758dfee764ae9c2a032e0151c6707',1,'transport.h']]],
   ['nc_5fsession_5faccept_5finout',['nc_session_accept_inout',['../db/d52/group__session.html#ga4d2da6dc354779dbd7a745147fa40457',1,'transport.h']]],
+  ['nc_5fsession_5faccept_5flibssh_5fchannel',['nc_session_accept_libssh_channel',['../d1/da1/libnetconf__ssh_8h.html#a159471d23a8253670ae176615f16965b',1,'libnetconf_ssh.h']]],
+  ['nc_5fsession_5faccept_5ftls',['nc_session_accept_tls',['../d5/d22/libnetconf__tls_8h.html#a9cc131efe272d1309653cb7cf445fa32',1,'libnetconf_tls.h']]],
   ['nc_5fsession_5faccept_5fusername',['nc_session_accept_username',['../db/d52/group__session.html#ga7ee207cac84fd5cd82419af41e14c06f',1,'transport.h']]],
   ['nc_5fsession_5fconnect',['nc_session_connect',['../db/d52/group__session.html#gae3858d998d19cb2fb700b15fdf602f23',1,'transport.h']]],
   ['nc_5fsession_5fconnect_5fchannel',['nc_session_connect_channel',['../db/d52/group__session.html#ga7d9e0f17bb4ca5b35c8343db59955060',1,'transport.h']]],
@@ -110,7 +113,7 @@ var searchData=
   ['nc_5fsession_5fsend_5freply',['nc_session_send_reply',['../d0/de2/group__reply.html#ga0b344811af933d80615f0874650938cc',1,'session.h']]],
   ['nc_5fsession_5fsend_5frpc',['nc_session_send_rpc',['../db/de9/group__rpc.html#gaeb0d93ea1b5ef0c22c9e5372425fb5c3',1,'session.h']]],
   ['nc_5fsession_5ftransport',['nc_session_transport',['../db/d52/group__session.html#ga6e125c035cdebae8c49ff962866c0806',1,'transport.h']]],
-  ['nc_5fset_5fkeypair_5fpath',['nc_set_keypair_path',['../db/d52/group__session.html#gaf3ce5316ea11c2f0359db16c2828ad6f',1,'callbacks_ssh.h']]],
+  ['nc_5fset_5fkeypair_5fpath',['nc_set_keypair_path',['../db/d52/group__session.html#gad067c538c9c44117a99cb0620e135108',1,'callbacks_ssh.h']]],
   ['nc_5fssh_5fpref',['nc_ssh_pref',['../db/d52/group__session.html#ga13119fdaa84544bf254bdb35ffdf83e0',1,'libnetconf_ssh.h']]],
   ['nc_5ftime2datetime',['nc_time2datetime',['../d3/d35/group__gen_a_p_i.html#ga90a44b60b04512fd0a3507ab35a1ccc4',1,'netconf.h']]],
   ['nc_5ftls_5fdestroy',['nc_tls_destroy',['../db/db4/group__tls.html#gacf3aed5cccac55d0548f46761707ece9',1,'libnetconf_tls.h']]],
@@ -131,7 +134,6 @@ var searchData=
   ['ncds_5fadd_5faugment_5ftransapi_5fstatic',['ncds_add_augment_transapi_static',['../d8/d55/group__transapi.html#ga7f6f5af711045d8598f283ec09b9167a',1,'datastore_xml.h']]],
   ['ncds_5fadd_5fmodel',['ncds_add_model',['../db/d67/group__store.html#ga145c9446fad8c350934c21ca86cbaf47',1,'datastore.h']]],
   ['ncds_5fadd_5fmodels_5fpath',['ncds_add_models_path',['../db/d67/group__store.html#ga93c65facb061f1048674343ac94a993b',1,'datastore.h']]],
-  ['ncds_5fapply_5frpc',['ncds_apply_rpc',['../db/d67/group__store.html#gaabfd0bf6abeaf40659043bf082e14801',1,'datastore.h']]],
   ['ncds_5fapply_5frpc2all',['ncds_apply_rpc2all',['../db/d67/group__store.html#ga6e6e979bd82e50913c82b4dc37cb8759',1,'datastore.h']]],
   ['ncds_5fbreak_5flocks',['ncds_break_locks',['../db/d67/group__store.html#ga4158273f1ae0332638cc9343097c9f0a',1,'datastore.h']]],
   ['ncds_5fconsolidate',['ncds_consolidate',['../db/d67/group__store.html#gab2a27448a52972db6807571590a5c6da',1,'datastore.h']]],
@@ -164,6 +166,7 @@ var searchData=
   ['ncntf_5fnotif_5fget_5fcontent',['ncntf_notif_get_content',['../da/d54/group__notifications.html#gadc78f4dd6440f4f1ac3ea12092946bd3',1,'notifications.h']]],
   ['ncntf_5fnotif_5fget_5ftime',['ncntf_notif_get_time',['../da/d54/group__notifications.html#gae00135a2cb85cb2d74d7f0b8b355ab47',1,'notifications.h']]],
   ['ncntf_5fnotif_5fget_5ftype',['ncntf_notif_get_type',['../da/d54/group__notifications.html#ga3f32038a98705a6dbb9d7038f67cbceb',1,'notifications.h']]],
+  ['ncntf_5fsession_5fget_5factive_5fsubscription',['ncntf_session_get_active_subscription',['../da/d54/group__notifications.html#ga3c44d0f8b0ffe29050de73f2ac209a32',1,'notifications.h']]],
   ['ncntf_5fstatus',['ncntf_status',['../da/d54/group__notifications.html#ga8c116b8aae2fe13811ebbfd3ca6f1a44',1,'notifications.h']]],
   ['ncntf_5fstream_5fallow_5fevents',['ncntf_stream_allow_events',['../da/d54/group__notifications.html#ga05a14f949bbb781fd58a233b1d2fab33',1,'notifications.h']]],
   ['ncntf_5fstream_5finfo',['ncntf_stream_info',['../da/d54/group__notifications.html#gaff62ddbbd893faf50d3db3fa3b08c21f',1,'notifications.h']]],
@@ -177,6 +180,7 @@ var searchData=
   ['ncxml_5ffilter_5fnew',['ncxml_filter_new',['../d7/dda/group__rpc__xml.html#gac05e048345a6130490bdbdde8e52a845',1,'messages_xml.h']]],
   ['ncxml_5freply_5fbuild',['ncxml_reply_build',['../d8/d73/group__reply__xml.html#gaaaf53b5a839be6935059fdf2818a5a42',1,'messages_xml.h']]],
   ['ncxml_5freply_5fdata',['ncxml_reply_data',['../d8/d73/group__reply__xml.html#ga1ce60f5416d4f75586836c1429c37149',1,'messages_xml.h']]],
+  ['ncxml_5freply_5fdata_5fns',['ncxml_reply_data_ns',['../d8/d73/group__reply__xml.html#ga3647f5a4bb6d8827a613a6b214d45581',1,'messages_xml.h']]],
   ['ncxml_5freply_5fdump',['ncxml_reply_dump',['../d8/d73/group__reply__xml.html#ga68ec8a0daf54f584339924bca5b5a682',1,'messages_xml.h']]],
   ['ncxml_5freply_5fget_5fdata',['ncxml_reply_get_data',['../d8/d73/group__reply__xml.html#ga7878bc6bc6b6d1ca002044d7edf85943',1,'messages_xml.h']]],
   ['ncxml_5frpc_5fbuild',['ncxml_rpc_build',['../d7/dda/group__rpc__xml.html#gaa6bcfc7c3e217e1b64c63bd674af4d8e',1,'messages_xml.h']]],
