@@ -495,6 +495,7 @@ NC_RPC_TYPE nc_rpc_parse_type(nc_rpc* rpc)
 static nc_rpc* _rpc_build(int libxml2, const struct nc_session* session, const char* str_dump, xmlDocPtr xml_dump)
 {
 	nc_rpc* rpc;
+	NC_OP op;
 
 	if (libxml2) {
 		assert(xml_dump);
@@ -508,11 +509,25 @@ static nc_rpc* _rpc_build(int libxml2, const struct nc_session* session, const c
 	}
 
 	/* assign operation value */
-	nc_rpc_assign_op(rpc);
+	op = nc_rpc_assign_op(rpc);
 
 	/* assign source/target datastore types */
-	nc_rpc_assign_ds(rpc, "source");
-	nc_rpc_assign_ds(rpc, "target");
+	if (op == NC_OP_GETCONFIG || op == NC_OP_COPYCONFIG || op == NC_OP_VALIDATE) {
+		if (!nc_rpc_assign_ds(rpc, "source")) {
+			ERROR("*%s: Missing <source> parameter of the RPC operation.", __func__);
+			nc_rpc_free(rpc);
+			return NULL;
+		}
+
+	}
+	if (op == NC_OP_EDITCONFIG || op == NC_OP_COPYCONFIG
+			|| op == NC_OP_DELETECONFIG || op == NC_OP_LOCK || op == NC_OP_UNLOCK) {
+		if (!nc_rpc_assign_ds(rpc, "target")) {
+			ERROR("*%s: Missing <target> parameter of the RPC operation.", __func__);
+			nc_rpc_free(rpc);
+			return NULL;
+		}
+	}
 
 	/* set rpc type flag */
 	nc_rpc_parse_type(rpc);
@@ -1897,11 +1912,11 @@ API nc_reply *nc_reply_data_ns(const char* data, const char* ns)
 	return (reply);
 }
 
-API nc_reply *ncxml_reply_data(const xmlNodePtr data)
+API nc_reply *ncxml_reply_data_ns(const xmlNodePtr data, const char* ns)
 {
 	nc_reply *reply;
 	xmlNodePtr content;
-	xmlNsPtr ns;
+	xmlNsPtr xmlns;
 
 	content = xmlNewNode(NULL, BAD_CAST "data");
 	if (content == NULL) {
@@ -1909,21 +1924,27 @@ API nc_reply *ncxml_reply_data(const xmlNodePtr data)
 		return (NULL);
 	}
 
-	if (xmlAddChildList(content, xmlCopyNodeList(data)) == NULL) {
+	if (data && xmlAddChildList(content, xmlCopyNodeList(data)) == NULL) {
 		ERROR("xmlAddChildList failed (%s:%d).", __FILE__, __LINE__);
 		xmlFreeNode(content);
 		return (NULL);
 	}
 
 	/* set namespace */
-	ns = xmlNewNs(content, (xmlChar *) NC_NS_BASE10, NULL);
-	xmlSetNs(content, ns);
+	xmlns = xmlNewNs(content, (xmlChar *) ns, NULL);
+	xmlSetNs(content, xmlns);
 
 	reply = (nc_reply*)nc_msg_create(content,"rpc-reply");
 	reply->type.reply = NC_REPLY_DATA;
 	xmlFreeNode(content);
 
 	return (reply);
+
+}
+
+API nc_reply *ncxml_reply_data(const xmlNodePtr data)
+{
+	return ncxml_reply_data_ns(data, NC_NS_BASE10);
 }
 
 static xmlNodePtr new_reply_error_content(struct nc_err* error)
