@@ -55,7 +55,7 @@
 #include <ctype.h>
 
 #ifndef DISABLE_LIBSSH
-#	include <libssh2.h>
+#	include <libssh/libssh.h>
 #endif
 
 #include <libxml/tree.h>
@@ -74,7 +74,8 @@
 #include "transport.h"
 
 #ifdef ENABLE_TLS
-# 	include "openssl/ssl.h"
+#	include <openssl/ssl.h>
+#	include <openssl/err.h>
 #endif
 
 #ifndef DISABLE_NOTIFICATIONS
@@ -164,8 +165,8 @@ static struct session_list_map *session_list = NULL;
 #else /* not DISABLE_LIBSSH */
 #ifdef ENABLE_TLS
 #define NC_WRITE(session,buf,c,ret) \
-	if(session->ssh_channel){ \
-		ret = libssh2_channel_write (session->ssh_channel, (buf), strlen(buf)); \
+	if(session->ssh_chan){ \
+		ret = ssh_channel_write (session->ssh_chan, (buf), strlen(buf)); \
 		if (ret > 0) {c += ret;} \
 	} else if (session->tls){ \
 		ret = SSL_write(session->tls, (buf), strlen(buf)); \
@@ -178,8 +179,8 @@ static struct session_list_map *session_list = NULL;
 	}
 #else /* not ENABLE_TLS */
 #define NC_WRITE(session,buf,c,ret) \
-	if(session->ssh_channel){ \
-		ret = libssh2_channel_write (session->ssh_channel, (buf), strlen(buf)); \
+	if(session->ssh_chan){ \
+		ret = ssh_channel_write (session->ssh_chan, (buf), strlen(buf)); \
 		if (ret > 0) {c += ret;} \
 	} else if (session->fd_output != -1) { \
 		ret = write (session->fd_output, (buf), strlen(buf)); \
@@ -1215,10 +1216,10 @@ void nc_session_close(struct nc_session* session, NC_SESSION_TERM_REASON reason)
 			}
 		} else
 #else
-		if (session->ssh_channel != NULL) {
+		if (session->ssh_chan != NULL) {
 			DBG_LOCK("mut_channel");
 			pthread_mutex_lock(session->mut_channel);
-			i = libssh2_channel_eof(session->ssh_channel);
+			i = ssh_channel_is_eof(session->ssh_chan);
 			DBG_UNLOCK("mut_channel");
 			pthread_mutex_unlock(session->mut_channel);
 			if (session->status == NC_SESSION_STATUS_WORKING &&  i == 0 && !session->is_server) {
@@ -1228,10 +1229,13 @@ void nc_session_close(struct nc_session* session, NC_SESSION_TERM_REASON reason)
 
 			DBG_LOCK("mut_channel");
 			pthread_mutex_lock(session->mut_channel);
-			libssh2_channel_free(session->ssh_channel);
+			/* server SSH channel, do not close or free */
+			if (!session->is_server) {
+				ssh_channel_free(session->ssh_chan);
+			}
+			session->ssh_chan = NULL;
 			DBG_UNLOCK("mut_channel");
 			pthread_mutex_unlock(session->mut_channel);
-			session->ssh_channel = NULL;
 		} else
 #endif
 #ifdef ENABLE_TLS
@@ -1239,23 +1243,28 @@ void nc_session_close(struct nc_session* session, NC_SESSION_TERM_REASON reason)
 			if (session->status == NC_SESSION_STATUS_WORKING && !session->is_server) {
 				announce_nc_session_closing(session);
 			}
-			SSL_shutdown(session->tls);
-			SSL_free(session->tls);
+			/* server TLS session, do not close or free */
+			if (!session->is_server) {
+				SSL_shutdown(session->tls);
+				SSL_free(session->tls);
+			}
 			session->tls = NULL;
 		}
 #else
 		{} /* close else which is not needed */
 #endif
 #ifndef DISABLE_LIBSSH
-		if (session->ssh_session != NULL && session->next == NULL && session->prev == NULL) {
-			/* close and free only if there is no other session using it */
-			libssh2_session_disconnect(session->ssh_session, nc_session_term_string(reason));
-			libssh2_session_free(session->ssh_session);
-			session->ssh_session = NULL;
+		if (!session->is_server) {
+			if (session->ssh_sess != NULL && session->next == NULL && session->prev == NULL) {
+				/* close and free only if there is no other session using it */
+				ssh_disconnect(session->ssh_sess);
+				ssh_free(session->ssh_sess);
+				session->ssh_sess = NULL;
 
-			close(session->transport_socket);
+				close(session->transport_socket);
+			}
+			session->transport_socket = -1;
 		}
-		session->transport_socket = -1;
 #endif
 
 		free(session->logintime);
@@ -1408,12 +1417,12 @@ static int nc_session_send(struct nc_session* session, struct nc_msg *msg)
 	struct pollfd fds;
 	int ret;
 
-	if ((session->fd_output == -1)
+	if (session->fd_output == -1 && session->transport_socket == -1
 #ifndef DISABLE_LIBSSH
-			&& (session->ssh_channel == NULL)
+			&& session->ssh_chan == NULL
 #endif
 #ifdef ENABLE_TLS
-			&& (session->tls == NULL)
+			&& session->tls == NULL
 #endif
 			&& 1) {
 		return (EXIT_FAILURE);
@@ -1430,7 +1439,24 @@ static int nc_session_send(struct nc_session* session, struct nc_msg *msg)
 
 	/* check that we are able to write data */
 	while (1) {
-		fds.fd = (session->transport_socket != -1) ? session->transport_socket : session->fd_output;
+		fds.fd = -1;
+
+		if (session->transport_socket != -1) {
+			fds.fd = session->transport_socket;
+		} else if (session->fd_output != -1) {
+			fds.fd = session->fd_output;
+		}
+#ifndef DISABLE_LIBSSH
+		else if (session->ssh_chan != NULL) {
+			fds.fd = ssh_get_fd(ssh_channel_get_session(session->ssh_chan));
+		}
+#endif
+#ifdef ENABLE_TLS
+		else if (session->tls != NULL) {
+			fds.fd = SSL_get_fd(session->tls);
+		}
+#endif
+
 		if (fds.fd == -1) {
 			ERROR("Invalid transport channel.");
 			nc_session_close(session, NC_SESSION_TERM_DROPPED);
@@ -1464,6 +1490,7 @@ static int nc_session_send(struct nc_session* session, struct nc_msg *msg)
 	DBG("Writing message (session %s): %s", session->session_id, text);
 
 	DBG_LOCK("mut_channel");
+	session->mut_channel_flag = 1;
 	pthread_mutex_lock(session->mut_channel);
 	/* if v1.1 send chunk information before message */
 	if (session->version == NETCONFV11) {
@@ -1471,19 +1498,25 @@ static int nc_session_send(struct nc_session* session, struct nc_msg *msg)
 		c = 0;
 		do {
 			NC_WRITE(session, &(buf[c]), c, ret);
-			if (ret < 0) {
-				DBG_UNLOCK("mut_channel");
-				pthread_mutex_unlock(session->mut_channel);
-				return (EXIT_FAILURE);
+			if (ret < 0 && (errno == EAGAIN || errno == EWOULDBLOCK)) {
+				usleep(10);
+				continue;
 			}
 #ifndef DISABLE_LIBSSH
-			if (c == LIBSSH2_ERROR_TIMEOUT) {
+			if (ret == SSH_ERROR) {
 				DBG_UNLOCK("mut_channel");
+				session->mut_channel_flag = 0;
 				pthread_mutex_unlock(session->mut_channel);
-				VERB("Writing data into the communication channel timeouted.");
+				VERB("Writing data into the communication channel failed.");
 				return (EXIT_FAILURE);
 			}
 #endif
+			if (ret < 0) {
+				DBG_UNLOCK("mut_channel");
+				session->mut_channel_flag = 0;
+				pthread_mutex_unlock(session->mut_channel);
+				return (EXIT_FAILURE);
+			}
 		} while (c < (ssize_t) strlen (buf));
 	}
 
@@ -1491,19 +1524,25 @@ static int nc_session_send(struct nc_session* session, struct nc_msg *msg)
 	c = 0;
 	do {
 		NC_WRITE(session, &(text[c]), c, ret);
-		if (ret < 0) {
-			DBG_UNLOCK("mut_channel");
-			pthread_mutex_unlock(session->mut_channel);
-			return (EXIT_FAILURE);
+		if (ret < 0 && (errno == EAGAIN || errno == EWOULDBLOCK)) {
+			usleep(10);
+			continue;
 		}
 #ifndef DISABLE_LIBSSH
-		if (c == LIBSSH2_ERROR_TIMEOUT) {
+		if (ret == SSH_ERROR) {
 			DBG_UNLOCK("mut_channel");
+			session->mut_channel_flag = 0;
 			pthread_mutex_unlock(session->mut_channel);
-			VERB("Writing data into the communication channel timeouted.");
+			VERB("Writing data into the communication channel failed.");
 			return (EXIT_FAILURE);
 		}
 #endif
+		if (ret < 0) {
+			DBG_UNLOCK("mut_channel");
+			session->mut_channel_flag = 0;
+			pthread_mutex_unlock(session->mut_channel);
+			return (EXIT_FAILURE);
+		}
 	} while (c < (ssize_t) strlen (text));
 	free (text);
 
@@ -1516,23 +1555,30 @@ static int nc_session_send(struct nc_session* session, struct nc_msg *msg)
 	c = 0;
 	do {
 		NC_WRITE(session, &(text[c]), c, ret);
-		if (ret < 0) {
-			DBG_UNLOCK("mut_channel");
-			pthread_mutex_unlock(session->mut_channel);
-			return (EXIT_FAILURE);
+		if (ret < 0 && (errno == EAGAIN || errno == EWOULDBLOCK)) {
+			usleep(10);
+			continue;
 		}
 #ifndef DISABLE_LIBSSH
-		if (c == LIBSSH2_ERROR_TIMEOUT) {
+		if (ret == SSH_ERROR) {
 			DBG_UNLOCK("mut_channel");
+			session->mut_channel_flag = 0;
 			pthread_mutex_unlock(session->mut_channel);
-			VERB("Writing data into the communication channel timeouted.");
+			VERB("Writing data into the communication channel failed.");
 			return (EXIT_FAILURE);
 		}
 #endif
+		if (ret < 0) {
+			DBG_UNLOCK("mut_channel");
+			session->mut_channel_flag = 0;
+			pthread_mutex_unlock(session->mut_channel);
+			return (EXIT_FAILURE);
+		}
 	} while (c < (ssize_t) strlen (text));
 
 	/* unlock the session's output */
 	DBG_UNLOCK("mut_channel");
+	session->mut_channel_flag = 0;
 	pthread_mutex_unlock(session->mut_channel);
 
 	return (EXIT_SUCCESS);
@@ -1540,9 +1586,6 @@ static int nc_session_send(struct nc_session* session, struct nc_msg *msg)
 
 static int nc_session_read_len(struct nc_session* session, size_t chunk_length, char **text, size_t *len)
 {
-#ifndef DISABLE_LIBSSH
-	char *err_msg;
-#endif
 	char *buf;
 	ssize_t c;
 	size_t rd = 0;
@@ -1566,21 +1609,20 @@ static int nc_session_read_len(struct nc_session* session, size_t chunk_length,
 
 	while (rd < chunk_length) {
 #ifndef DISABLE_LIBSSH
-		if (session->ssh_channel) {
-			/* read via libssh2 */
-			c = libssh2_channel_read(session->ssh_channel, &(buf[rd]), chunk_length - rd);
-			if (c == LIBSSH2_ERROR_EAGAIN || c == LIBSSH2_ERROR_TIMEOUT) {
+		if (session->ssh_chan) {
+			/* read via libssh */
+			c = ssh_channel_read(session->ssh_chan, &(buf[rd]), chunk_length - rd, 0);
+			if (c == SSH_AGAIN) {
 				usleep (NC_READ_SLEEP);
 				continue;
-			} else if (c < 0) {
-				libssh2_session_last_error (session->ssh_session, &err_msg, NULL, 0);
-				ERROR("Reading from the SSH channel failed (%zd: %s)", c, err_msg);
+			} else if (c == SSH_ERROR) {
+				ERROR("Reading from the SSH channel failed (%zd: %s)", ssh_get_error_code(session->ssh_sess), ssh_get_error(session->ssh_sess));
 				free (buf);
 				*len = 0;
 				*text = NULL;
 				return (EXIT_FAILURE);
 			} else if (c == 0) {
-				if (libssh2_channel_eof (session->ssh_channel)) {
+				if (ssh_channel_is_eof(session->ssh_chan)) {
 					ERROR("Server has closed the communication socket");
 					free (buf);
 					*len = 0;
@@ -1646,9 +1688,6 @@ static int nc_session_read_len(struct nc_session* session, size_t chunk_length,
 
 static int nc_session_read_until(struct nc_session* session, const char* endtag, unsigned int limit, char **text, size_t *len)
 {
-#ifndef DISABLE_LIBSSH
-	char *err_msg;
-#endif
 	size_t rd = 0;
 	ssize_t c;
 	char *buf = NULL;
@@ -1682,15 +1721,18 @@ static int nc_session_read_until(struct nc_session* session, const char* endtag,
 			return (EXIT_FAILURE);
 		}
 #ifndef DISABLE_LIBSSH
-		if (session->ssh_channel) {
-			/* read via libssh2 */
-			c = libssh2_channel_read(session->ssh_channel, &(buf[rd]), 1);
-			if (c == LIBSSH2_ERROR_EAGAIN || c == LIBSSH2_ERROR_TIMEOUT) {
+		if (session->ssh_chan) {
+			/* read via libssh */
+			c = ssh_channel_read(session->ssh_chan, &(buf[rd]), 1, 0);
+			if (c == SSH_AGAIN) {
 				usleep (NC_READ_SLEEP);
 				continue;
-			} else if (c < 0) {
-				libssh2_session_last_error (session->ssh_session, &err_msg, NULL, 0);
-				ERROR("Reading from the SSH channel failed (%zd: %s)", c, err_msg);
+			} else if (c == SSH_ERROR) {
+				if (session->ssh_sess != NULL) {
+					ERROR("Reading from the SSH channel failed (%zd: %s)", ssh_get_error_code(session->ssh_sess), ssh_get_error(session->ssh_sess));
+				} else {
+					ERROR("Reading from the SSH channel failed");
+				}
 				free (buf);
 				if (len != NULL) {
 					*len = 0;
@@ -1700,7 +1742,7 @@ static int nc_session_read_until(struct nc_session* session, const char* endtag,
 				}
 				return (EXIT_FAILURE);
 			} else if (c == 0) {
-				if (libssh2_channel_eof (session->ssh_channel)) {
+				if (ssh_channel_is_eof (session->ssh_chan)) {
 					ERROR("Server has closed the communication socket");
 					free (buf);
 					if (len != NULL) {
@@ -1725,10 +1767,20 @@ static int nc_session_read_until(struct nc_session* session, const char* endtag,
 					usleep(NC_READ_SLEEP);
 					continue;
 				} else {
-					ERROR("Reading from the TLS session failed (%d)", r);
+					if (r == SSL_ERROR_SYSCALL) {
+						ERROR("Reading from the TLS session failed (%s)", strerror(errno));
+					} else if (r == SSL_ERROR_SSL) {
+						ERROR("Reading from the TLS session failed (%s)", ERR_error_string(r, NULL));
+					} else {
+						ERROR("Reading from the TLS session failed (SSL code %d)", r);
+					}
 					free (buf);
-					*len = 0;
-					*text = NULL;
+					if (len != NULL) {
+						*len = 0;
+					}
+					if (text != NULL) {
+						*text = NULL;
+					}
 					return (EXIT_FAILURE);
 				}
 			}
@@ -1872,50 +1924,40 @@ static NC_MSG_TYPE nc_session_receive(struct nc_session* session, int timeout, s
 	NC_MSG_TYPE msgtype;
 	xmlNodePtr root;
 
-#ifndef DISABLE_LIBSSH
-	LIBSSH2_POLLFD fds_ssh;
-#endif
-
 	if (session == NULL || (session->status != NC_SESSION_STATUS_WORKING && session->status != NC_SESSION_STATUS_CLOSING)) {
 		ERROR("Invalid session to receive data.");
 		return (NC_MSG_UNKNOWN);
 	}
 
+	/* if there is waiting sending thread, sleep for timeout and return
+	 * with NC_MSG_WOULDBLOCK because we probably relock the mut_channel quite
+	 * fast, so we are trying to avoid starvation this way
+	 */
+	if (session->mut_channel_flag) {
+		usleep(timeout ? timeout : 1);
+		return (NC_MSG_WOULDBLOCK);
+	}
+
 	/* lock the session for receiving */
 	DBG_LOCK("mut_channel");
 	pthread_mutex_lock(session->mut_channel);
 
 	/* use while for possibility of repeating test */
 	while(1) {
+		revents = 0;
 #ifndef DISABLE_LIBSSH
-		if (session->ssh_channel != NULL) {
+		if (session->ssh_chan != NULL) {
 			/* we are getting data from libssh's channel */
-			fds_ssh.type = LIBSSH2_POLLFD_CHANNEL;
-			fds_ssh.fd.channel = session->ssh_channel;
-			fds_ssh.events = LIBSSH2_POLLFD_POLLIN;
-			fds_ssh.revents = LIBSSH2_POLLFD_POLLIN;
-			/*
-			 * According to libssh2 documentation, standard poll should work, but it does not.
-			 * Using standard poll, we are not able to detect data on a specific channel, it
-			 * is possible only using libssh2_poll(). Unfortunatelly, using libssh2_poll()
-			 * we are not able to detect closed socket and close it either, so it can remain
-			 * in CLOSE_WAIT status.
-			 */
-			/*
-			fds.fd = nc_session_get_eventfd(session);
-			fds.events = POLLIN;
-			fds.revents = 0;
-			status = poll(&fds, 1, timeout);
-			*/
-			status = libssh2_poll(&fds_ssh, 1, timeout);
-
-			revents = fds_ssh.revents;
+			status = ssh_channel_poll_timeout(session->ssh_chan, timeout, 0);
+			if (status > 0) {
+				revents = POLLIN;
+			}
 		} else
 #endif
 #ifdef ENABLE_TLS
 		if (session->tls != NULL) {
 			/* we are getting data from TLS session using OpenSSL */
-			fds.fd = nc_session_get_eventfd(session);
+			fds.fd = SSL_get_fd(session->tls);
 			fds.events = POLLIN;
 			fds.revents = 0;
 			status = poll(&fds, 1, timeout);
@@ -2058,6 +2100,11 @@ static NC_MSG_TYPE nc_session_receive(struct nc_session* session, int timeout, s
 	DBG_UNLOCK("mut_channel");
 	pthread_mutex_unlock(session->mut_channel);
 
+	if (text == NULL) {
+		ERROR("Empty message received (session %s)", session->session_id);
+		goto malformed_msg;
+	}
+
 	retval = calloc (1, sizeof(struct nc_msg));
 	if (retval == NULL) {
 		ERROR("Memory reallocation failed (%s:%d).", __FILE__, __LINE__);
@@ -2156,7 +2203,7 @@ malformed_msg_channels_unlock:
 	pthread_mutex_unlock(session->mut_channel);
 
 malformed_msg:
-	if (session->version == NETCONFV11 && session->ssh_session == NULL) {
+	if (session->version == NETCONFV11 && session->ssh_sess == NULL) {
 		/* NETCONF version 1.1 define sending error reply from the server */
 		reply = nc_reply_error(nc_err_new(NC_ERR_MALFORMED_MSG));
 		if (reply == NULL) {
@@ -2165,7 +2212,11 @@ malformed_msg:
 			return (NC_MSG_UNKNOWN);
 		}
 
-		nc_session_send_reply(session, NULL, reply);
+		if (nc_session_send_reply(session, NULL, reply) == 0) {
+			ERROR("Unable to send the \'Malformed message\' reply");
+			nc_session_close(session, NC_SESSION_TERM_OTHER);
+			return (NC_MSG_UNKNOWN);
+		}
 		nc_reply_free(reply);
 	}
 
@@ -2400,8 +2451,10 @@ try_again:
 API NC_MSG_TYPE nc_session_recv_rpc(struct nc_session* session, int timeout, nc_rpc** rpc)
 {
 	NC_MSG_TYPE ret;
+	NC_OP op;
 	struct nc_err* e = NULL;
 	nc_reply* reply;
+	unsigned int *counter = NULL;
 
 	/* use local timeout to avoid continual long time blocking */
 	int local_timeout;
@@ -2470,21 +2523,8 @@ try_again:
 			}
 
 			if (e != NULL) {
-				reply = nc_reply_error(e);
-				nc_session_send_reply(session, *rpc, reply);
-				nc_rpc_free(*rpc);
-				*rpc = NULL;
-				nc_reply_free(reply);
-
-				/* update stats */
-				session->stats->in_bad_rpcs++;
-				if (nc_info) {
-					pthread_rwlock_wrlock(&(nc_info->lock));
-					nc_info->stats.counters.in_bad_rpcs++;
-					pthread_rwlock_unlock(&(nc_info->lock));
-				}
-
-				return (NC_MSG_NONE); /* message processed internally */
+				counter = &nc_info->stats.counters.in_bad_rpcs;
+				goto replyerror;
 			}
 		}
 		/* update statistics */
@@ -2502,30 +2542,42 @@ try_again:
 		if (nacm_check_operation(*rpc) != NACM_PERMIT) {
 			e = nc_err_new(NC_ERR_ACCESS_DENIED);
 			nc_err_set(e, NC_ERR_PARAM_MSG, "Operation not permitted.");
-			reply = nc_reply_error(e);
-			nc_session_send_reply(session, *rpc, reply);
-			nc_rpc_free(*rpc);
-			*rpc = NULL;
-			nc_reply_free(reply);
-			/* update stats */
-			if (nc_info) {
-				pthread_rwlock_wrlock(&(nc_info->lock));
-				nc_info->stats_nacm.denied_ops++;
-				pthread_rwlock_unlock(&(nc_info->lock));
-			}
-
-			return (NC_MSG_NONE); /* message processed internally */
+			counter = &nc_info->stats_nacm.denied_ops;
+			goto replyerror;
 		}
 
 		/* assign operation value */
-		nc_rpc_assign_op(*rpc);
+		op = nc_rpc_assign_op(*rpc);
 
 		/* set rpc type flag */
 		nc_rpc_parse_type(*rpc);
 
 		/* assign source/target datastore types */
-		nc_rpc_assign_ds(*rpc, "source");
-		nc_rpc_assign_ds(*rpc, "target");
+		if (op == NC_OP_GETCONFIG || op == NC_OP_COPYCONFIG || op == NC_OP_VALIDATE) {
+			nc_rpc_assign_ds(*rpc, "source");
+			if (!(*rpc)->source) {
+				e = nc_err_new(NC_ERR_MISSING_ELEM);
+				nc_err_set(e, NC_ERR_PARAM_TYPE, "protocol");
+				nc_err_set(e, NC_ERR_PARAM_INFO_BADELEM, "source");
+
+				session->stats->in_bad_rpcs++;
+				counter = &nc_info->stats.counters.in_bad_rpcs;
+				goto replyerror;
+			}
+		}
+		if (op == NC_OP_EDITCONFIG || op == NC_OP_COPYCONFIG
+				|| op == NC_OP_DELETECONFIG || op == NC_OP_LOCK || op == NC_OP_UNLOCK) {
+			nc_rpc_assign_ds(*rpc, "target");
+			if (!(*rpc)->target) {
+				e = nc_err_new(NC_ERR_MISSING_ELEM);
+				nc_err_set(e, NC_ERR_PARAM_TYPE, "protocol");
+				nc_err_set(e, NC_ERR_PARAM_INFO_BADELEM, "target");
+
+				session->stats->in_bad_rpcs++;
+				counter = &nc_info->stats.counters.in_bad_rpcs;
+				goto replyerror;
+			}
+		}
 
 		break;
 	case NC_MSG_HELLO:
@@ -2551,6 +2603,24 @@ try_again:
 	}
 
 	return (ret);
+
+replyerror:
+
+	reply = nc_reply_error(e);
+	if (nc_session_send_reply(session, *rpc, reply) == 0) {
+		ERROR("Failed to send reply.");
+	}
+	nc_rpc_free(*rpc);
+	*rpc = NULL;
+	nc_reply_free(reply);
+	/* update stats */
+	if (nc_info) {
+		pthread_rwlock_wrlock(&(nc_info->lock));
+		(*counter)++;
+		pthread_rwlock_unlock(&(nc_info->lock));
+	}
+
+	return (NC_MSG_NONE); /* message processed internally */
 }
 
 API const nc_msgid nc_session_send_rpc(struct nc_session* session, nc_rpc *rpc)
